unit Processing;

interface

{$MINENUMSIZE 4}

uses Windows, Common, IniFiles;

const
  szPluginName = ' ';
  szPluginDesc = '    (  MMX)';

type
  TSettings = record
    MyPar1 : Integer;
    MyPar2 : Boolean;
  end;

var
  g_hParent   : HWND = 0;
  g_Settings  : TSettings;

  function BeholdTV_Plugin_Version() : ULONG; cdecl;
  function BeholdTV_Plugin_PR() : PPROC_PLUGIN; cdecl;

  procedure PP_Init(pPrInit : PINIT_STR); cdecl;
  function  PP_StartStreaming(pPrPar : PPARAM_STR) : BOOL; cdecl;
  function  PP_Inverse(pPr : PPROC_STR): BOOL; cdecl;
  procedure PP_StopStreaming(); cdecl;
  procedure PP_ShowDialog(pDlgPar : PDLG_STR); cdecl;
  procedure PP_Uninit(); cdecl;

  procedure GetIniFileName();
  procedure LoadPluginSettings();
  procedure SavePluginSettings();

implementation

uses
  Controls, MyDialog;

var
  ConfigForm      : TConfigForm = Nil;
  g_InversePlugin : PROC_PLUGIN;     //    
  g_szIniFile     : String = '';
  g_nFrameWidth   : Longint;
  g_nFrameHeight  : Longint;
  g_nBitPerPixel  : Longint;
  g_PixFmt        : ePixFmt;
  g_ulTypeCPU     : ULONG;
  g_AlreadyLoad   : Boolean = False;
  g_PtrByOrd      : BY_ORD_FUNC = Nil;
  g_PtrByIdx      : BY_IDX_FUNC = Nil;


// ****************   () **************************

//       ,    
procedure GetIniFileName();
Var
  szPathFull : array[0..MAX_PATH - 1] of Char;
  szPathIni  : array[0..MAX_PATH - 1] of Char;
  pszFile    : PChar;
  szPath     : String;
Begin
  if (g_szIniFile <> '') then Exit;

  if ( GetModuleFileName(HInstance, szPathFull, sizeof(szPathFull)) <> 0) then begin
    if (GetFullPathName(szPathFull, sizeof(szPathIni), szPathIni, pszFile) <> 0) then begin
      szPath := String(szPathIni);
      g_szIniFile := Copy(szPath, 1, Length(szPath) - 4) + '.ini';
    end;
  end;
  
End;


procedure LoadPluginSettings();
Var
  MyIni : TIniFile;
Begin
  if (g_AlreadyLoad) then Exit;
  
  GetIniFileName();

  //       Preview  Capture ( )
  if (g_szIniFile <> '') then begin
    MyIni := TIniFile.Create(g_szIniFile);
    try
      g_Settings.MyPar1 := MyIni.ReadInteger ('Common', 'MyPar1', 640);
      g_Settings.MyPar2 := MyIni.ReadBool ('Common', 'MyPar2', True);

      g_AlreadyLoad := True;
    finally
      MyIni.Free();
    end;
  end;

  // :
  //   VCL  TIniFile,  dll   ~80 
  //      TIniFile,  WinAPI  
  // ini  "GetPrivateProfileInt"  "GetPrivateProfileString"

End;

procedure SavePluginSettings();
Var
  MyIni : TIniFile;
Begin
  GetIniFileName();

  //       Preview  Capture ( )
  if (g_szIniFile <> '') then begin
    MyIni := TIniFile.Create(g_szIniFile);
    try
      MyIni.WriteInteger ('Common', 'MyPar1', g_Settings.MyPar1);
      MyIni.WriteBool ('Common', 'MyPar2', g_Settings.MyPar2);
    finally
      MyIni.Free();
    end;
  end;

  // :
  //   VCL  TIniFile,  dll   ~80 
  //      TIniFile,  WinAPI  
  //  ini  "WritePrivateProfileString"

End;

// ******************   (  API) *********************

////////////////////////////////////////////////////////////////////////////////
// *   *
////////////////////////////////////////////////////////////////////////////////
procedure PP_Init(pPrInit : PINIT_STR);
Begin
 //     

 //        
  if (pPrInit <> Nil) then begin
    g_PtrByOrd := pPrInit^.pChByOrd;
    g_PtrByIdx := pPrInit^.pChByIdx;
  end;  

  LoadPluginSettings();  //    
End;

////////////////////////////////////////////////////////////////////////////////
// *  StartStreaming *
////////////////////////////////////////////////////////////////////////////////
function PP_StartStreaming(pPrPar : PPARAM_STR) : BOOL;
Begin
 //     
 //   ,  CPU ( )
 // ( ,       ..)

  if (pPrPar <> Nil) then begin
    g_nFrameWidth  := pPrPar^.FrameWidth;
    g_nFrameHeight := pPrPar^.FrameHeight;
    g_nBitPerPixel := pPrPar^.BitPerPixel;
    g_PixFmt      := pPrPar^.PixFmt;
    g_ulTypeCPU    := pPrPar^.TypeCPU;
  end;

 //..........................................

 Result := TRUE;
End;


////////////////////////////////////////////////////////////////////////////////
// *   *
////////////////////////////////////////////////////////////////////////////////
function PP_Inverse(pPr : PPROC_STR): BOOL;
Const
  InvertMask64 : int64 = $ffffffffffffffff;
  InvertMask32 : ULONG = $ffffffff;
Var
  x, y, xcycles : Longint;
  pSrcBuffer    : PBYTE;
  Block32       : PULONG;
  NameTV        : String;
  pInfoTV       : PChannelInfo;
Begin
  // .       
  //    

  Result := FALSE;

  pSrcBuffer := pPr^.SrcBuff[0];         //        
  if (pSrcBuffer = Nil) then             //    
    Exit;

  //    ,    ,  CPU    MMX

  if (g_ulTypeCPU <> CPU_SIMPLE) then begin

    xcycles := pPr^.SrcLength shr 3;               // (pPr^.SrcLength / 8)

    for y := 0 to pPr^.FrameHeight - 1 do begin    //    
      for x := 0 to xcycles - 1 do begin           //  1/8   

        asm
          MOV    eax, dword ptr[pSrcBuffer]        //   eax    
          MOVQ   mm0, qword ptr[eax]               //   MMX  mm0    8 
          PXOR   mm0, InvertMask64                 //    MMX  mm0
          MOVQ   qword ptr[eax], mm0               //    MMX   8   
          ADD    pSrcBuffer, 8                     //      8 
        end;

      end;
    end;

    asm
      emms                                         //     MMX
    end;

  end
    else begin   //    MMX

      xcycles := pPr^.SrcLength shr 2;             // (pPr^.SrcLength / 4)

      for y := 0 to pPr^.FrameHeight - 1 do begin  //    
        for x := 0 to xcycles - 1 do begin         //  1/4   

          Block32  := PULONG(pSrcBuffer);          //    32-  (4   )
          Block32^ := Block32^ xor InvertMask32;   //    32  
          inc(pSrcBuffer, 4);                      //      4 

        end;
      end;

    end;


  //   ,    :

  //     
  NameTV := pPr^.ChInfo.ChnlName;

  //       = 8
  pInfoTV := g_PtrByOrd(8);

  if (pInfoTV <> Nil) then
    NameTV := String(pInfoTV.ChnlName);

  //       = 127358376293300000
  pInfoTV := g_PtrByIdx(127358376293300000);

  if (pInfoTV <> Nil) then
    NameTV := String(pInfoTV.ChnlName);

  Result := TRUE;
End;


////////////////////////////////////////////////////////////////////////////////
// *  StopStreaming *
////////////////////////////////////////////////////////////////////////////////
procedure PP_StopStreaming();
Begin
 //     
 // (      ..)


End;


////////////////////////////////////////////////////////////////////////////////
// *     *
////////////////////////////////////////////////////////////////////////////////
procedure PP_ShowDialog(pDlgPar : PDLG_STR);
Begin
  g_hParent := pDlgPar^.hwndParent;

  LoadPluginSettings();  //   

  if (ConfigForm = nil) then
    ConfigForm := TConfigForm.Create(Nil);

  ConfigForm.Show();

  // :
  //   VCL  TForm,  dll   ~350 
  //      TForm,  WinAPI  
  //  "CreateDialog", "DialogBox",       .
  
End;


////////////////////////////////////////////////////////////////////////////////
// *   *
////////////////////////////////////////////////////////////////////////////////
procedure PP_Uninit();
Begin
 //     

  SavePluginSettings();  //    

  if (ConfigForm <> nil) then begin
    ConfigForm.Close();
    ConfigForm.Free();
    ConfigForm := Nil;    
  end;

  g_PtrByOrd := Nil;
  g_PtrByIdx := Nil;

End;


// ********************* Exports ***********************************************

function BeholdTV_Plugin_Version() : ULONG;
Begin
  Result := DSPLUGINVER;
End;

function BeholdTV_Plugin_PR() : PPROC_PLUGIN;
Begin
  g_InversePlugin.szName        := PChar(szPluginName);
  g_InversePlugin.szDesc        := PChar(szPluginDesc);
  g_InversePlugin.pPluginInit   := PP_Init;            //     (  Nil)
  g_InversePlugin.pPluginStart  := PP_StartStreaming;  //    StartStreaming (  Nil)
  g_InversePlugin.pPluginProc   := PP_Inverse;         //     ()
  g_InversePlugin.pPluginStop   := PP_StopStreaming;   //    StopStreaming (  Nil)
  g_InversePlugin.pPluginDialog := PP_ShowDialog;      //       (  Nil)
  g_InversePlugin.pPluginUninit := PP_Uninit;          //     (  Nil)

  Result := @g_InversePlugin;
End;

end.
